@@ -37,7 +37,6 @@
 #include <locale.h>
 #endif
 
-#define JNLIB_NEED_LOG_LOGV
 #include "i18n.h"
 #include "util.h"
 #include "exechelp.h"
@@ -86,7 +85,7 @@ my_libassuan_log_handler (assuan_context_t ctx, void *hook,
     return 0; /* Assuan debugging is not enabled.  */
 
   if (msg)
-    log_string (JNLIB_LOG_DEBUG, msg);
+    log_string (GPGRT_LOG_DEBUG, msg);
 
   return 1;
 }
